class FhirMergeResponseSchema:
    """
    This class provides names for columns in FhirMergeResponse

    Should match to https://github.com/icanbwell/helix.fhir.client.sdk/blob/main/helix_fhir_client_sdk/responses/fhir_merge_response.py
    """

    request_id = "request_id"
    url = "url"
    responses = "responses"
    error = "error"
    access_token = "access_token"
    status = "status"
    data = "data"
